// powsim.C  Version 7.42  Copyleft (c) 杉本典夫
// Time-stamp: <2023-12-15 09:17:28 tac>
//
// シミュレーションによって検出力を計算する。
//
// 使用ライブラリ:libdans.so, libfun.so
// コンパイル法  :g++ -O -s -W powsim powsim.C -I/usr/local/include
//               -L/usr/local/lib -ldans -lfun

#include <dans.H>               // dans用ヘッダファイル(mat.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法
{
  "powsim  Version 7.42         Copyleft (Vo\\oV) 2023 杉本典夫",
  "書式:powsim  [<オプション> …]  [<データファイル> …]",
  "機能:シミュレーションによって検出力を計算する。",
  "<オプション>",
  "  -a[=<有意水準>]:有意水準αを指定する。",
  "    (0<有意水準α<1,オプション省略時:-a=0.05)",
  "  -am[={-|+}]:ANOVAが有意の時だけ多重比較実施。",
  "    (-am:-am=+,オプション省略時:-am=-)",
  "  -f=<ファイル名>:指定されたファイルからオプションを読み込む。",
  "  -g[=<群数>]:群の数を指定する。(1<群数,オプション省略時:-g=3)",
  "  -h[=<コマンド名>]:オンラインマニュアルを表示する。(-h:-h=man)",
  "  -m=<平均値1>[,<平均値2>,…]:群の平均値を指定する。",
  "    指定しない群の平均値は0になる。",
  "  -n=<例数1>[,<例数2>,…]:群の例数を指定する。(1<例数)",
  "    指定しない群の例数は100になる。",
  "  -r[=<繰り返し数>]:繰り返し数を指定する。",
  "    (0<繰り返し数,オプション省略時:-r=10000)",
  "  -sd=<標準偏差1>,[<標準偏差2>,…]群の標準偏差を指定する。(0<標準偏差)",
  "    指定しない群の標準偏差は1になる。",
  "  -seed[=<整数>]乱数のタネを指定する。(0<整数,オプション省略時:-seed=1)",
  "  -v[={-|+}]:詳しい情報を出力する。(-v:-v=+,オプション省略時:-v=-)"
};

const string Title = "=== シミュレーションによる検出力の計算 ==="; // 表題
constexpr int Gdef = 3;         // 省略時群数
constexpr double Mdef = 0.0;    // 省略時群の平均値
constexpr int Ndef = 100;       // 省略時群の例数
constexpr int Rdef = 10000;     // 省略時繰り返し数
constexpr double SDdef = 1.0;   // 省略時群の標準偏差
constexpr int SEEDdef = 1;      // 省略時乱数のタネ

class option                    // オプションクラス
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  double va;                    // 有意水準
  bool vam;                     // ANOVA&多重比較フラッグ
  int vg;                       // 群数
  lstr mlst;                    // 平均値リスト
  vecdbl vmvec;                 // 平均値ベクトル
  lstr nlst;                    // 例数リスト
  vecint vnvec;                 // 例数ベクトル
  int vr;                       // 繰り返し数
  lstr sdlst;                   // 標準偏差リスト
  vecdbl vsdvec;                // 標準偏差ベクトル
  int vseed;                    // 乱数のタネ
  bool vv;                      // 詳しい情報の出力フラッグ
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  double a() const              // 有意水準を返す
  { return va; }
  bool am() const               // ANOVA&多重比較フラッグを返す
  { return vam; }
  double m(size_t idx) const    // 検出群の平均値を返す
  { return (idx < vmvec.size()) ? vmvec[idx] : 0.0; }
  int n(size_t idx) const       // 群の例数を返す
  { return (idx < vnvec.size()) ? vnvec[idx] : 0; }
  int g() const                 // 群数を返す
  { return vg; }
  int r() const                 // 繰り返し数を返す
  { return vr; }
  double sd(size_t idx) const   // 群の標準偏差を返す
  { return (idx < vsdvec.size()) ? vsdvec[idx] : 0.0; }
  int seed() const              // 乱数のタネを返す
  { return vseed; }
  bool v() const                // 詳しい情報出力フラッグを返す
  { return vv; }
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check();                 // オプションのチェックとセット
};

option::option(const char* c_pname) : vpath(), vpname(), va(ALdef), vam(false), vg(Gdef), mlst(), vmvec(), nlst(), vnvec(), vr(Rdef), sdlst(), vsdvec(), vseed(SEEDdef), vv(false)
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "a")            // 有意水準の指定
  {
    va = opt[1].empty() ? ALdef : strtodf(opt[1]);
    if (va <= 0 || 1.0 <= va) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "am")      // ANOVA&多重比較指定
  {
    if (opt[1].empty() || opt[1] == "+") vam = true;
    else if (opt[1] == "-") vam = false;
    else err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "g")      // 群数の指定
  {
    char* ptr = nullptr;
    vg = opt[1].empty() ? Gdef : strtol(opt[1].c_str(), &ptr, 0);
    if (vg < 2) err = badopt(vpname, optstr, USAGE, 2.0, (double)UCHAR_MAX);
  }
  else if (opt[0] == "m")       // 群の平均値の指定
  {
    if (strsplit(mlst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "n")       // 群の例数の指定
  {
    if (strsplit(nlst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "r")       // 繰り返し数の指定
  {
    char* ptr = nullptr;
    vr = opt[1].empty() ? Rdef : strtol(opt[1].c_str(), &ptr, 0);
    if (vr < 1) err = badopt(vpname, optstr, USAGE, 1.0, (double)INT_MAX);
  }
  else if (opt[0] == "sd")      // 群の標準偏差の指定
  {
    if (strsplit(sdlst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "seed")    // 乱数のタネの指定
  {
    char* ptr = nullptr;
    vseed = opt[1].empty() ? SEEDdef : strtol(opt[1].c_str(), &ptr, 0);
    if (vseed < 1) err = badopt(vpname, optstr, USAGE, 1.0, (double)RAND_MAX);
  }
  else if (opt[0] == "v")       // 詳しい情報出力指定
  {
    if (opt[1].empty() || opt[1] == "+") vv = true;
    else if (opt[1] == "-") vv = false;
    else err = badopt(vpname, optstr, USAGE);
  }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("ヘルプ表示");
  if (err < 0) throw option_error(string("オプション指定違反→") + optstr);
}

int option::check()             // オプションのチェックとセット
{
  vmvec.resize(vg, Mdef);  vnvec.resize(vg, Ndef);  vsdvec.resize(vg, SDdef);
  if (mlst.size() > 0)
  {
    int i = 0;
    lstr_citr pos = mlst.begin();
    while (pos != mlst.end() && i < vg)
    {
      vmvec[i] = strtodf(*pos);
      if (vmvec[i] == BLANKf) return disperr(vpname, "平均値指定が不適切！");
      else { ++i;  ++pos; }
    }
  }
  if (nlst.size() > 0)
  {
    int i = 0;
    lstr_citr pos = nlst.begin();
    while (pos != nlst.end() && i < vg)
    {
      char* ptr = 0;
      vnvec[i] = strtol(pos->c_str(), &ptr, 0);
      if (vnvec[i] < 1) return disperr(vpname, "例数指定が不適切！");
      else { ++i;  ++pos; }
    }
  }
  if (sdlst.size() > 0)
  {
    int i = 0;
    lstr_citr pos = sdlst.begin();
    while (pos != sdlst.end() && i < vg)
    {
      vsdvec[i] = strtodf(*pos);
      if (sgn(vsdvec[i]) <= 0 || vsdvec[i] == BLANKf) return disperr(vpname, "標準偏差指定が不適切！");
      else { ++i;  ++pos; }
    }
  }
  return 0;                     // エラーなし
}

class varclass                  // 変数クラス
{
private:
  int vg;                    // 群数
  int vnc;                   // 多重比較の比較回数
  int vnasig;                // 分散分析で有意になった回数
  int vnfsig;                // Fisher型多重比較で有意になった回数
  int vnbsig;                // Bonferroni型多重比較で有意になった回数
  int vnssig;                // Scheffe型多重比較で有意になった回数
  vector<statk<double>> vstativec; // 1回ごとの統計量
  vector<statk<double>> vstattvec; // 標本平均値の統計量
  mat<double> vsmat;               // t値の相関計算用行列
  int vnt;                         // t値の例数
  vecdbl vtvec;                    // t値ベクトル
  vecdbl vmevec;                   // t値の平均値ベクトル
  vecdbl vvrvec;                   // t値の分散ベクトル
  void set_stat(const option&opt);
  // 正規乱数を発生して基礎統計量を集計する
  double cal_anova(double& vr) const; // 分散分析を行う
  bool cal_mcomp(bool& fsig, bool& bsig, bool& ssig, double vr, double alpha);
  // 多重比較を行う
public:
  varclass(const option& opt);  // コンストラクタ
  ~varclass()                   // デストラクタ
  {}
  void set(const option& opt);  // 変数をセットする
  int n(size_t idx) const       // 標本平均値の例数を返す
  { return (idx < vstattvec.size()) ? vstattvec[idx].n() : 0; }
  double mean(size_t idx) const // 標本平均値の平均値を返す
  { return (idx < vstattvec.size()) ? vstattvec[idx].mean() : 0.0; }
  double vr(size_t idx) const   // 標本平均値の不偏分散を返す
  { return (idx < vstattvec.size()) ? vstattvec[idx].vr() : 0.0; }
  double b1popu(size_t idx) const; // 標本平均値の母歪度を返す
  double b2popu(size_t idx) const; // 標本平均値の母尖度を返す
  int nt() const                // t値の例数を返す
  { return vnt; }
  const mat<double> smat() const // t値の相関計算用行列を返す
  { return vsmat; }
  const vecdbl& mevec() const   // t値の平均値ベクトルを返す
  { return vmevec; }
  const vecdbl& vrvec() const   // t値の分散ベクトルを返す
  { return vvrvec; }
  int nasig() const             // 分散分析で有意になった回数を返す
  { return vnasig; }
  int nfsig() const             // Fisher型多重比較で有意になった回数を返す
  { return vnfsig; }
  int nbsig() const             // Bonferroni型多重比較で有意になった回数を返す
  { return vnbsig; }
  int nssig() const             // Scheffe型多重比較で有意になった回数を返す
  { return vnssig; }
};

varclass::varclass(const option&opt) : vg(0), vnasig(0), vnfsig(0), vnbsig(0), vnssig(0), vstativec(), vstattvec(), vsmat(), vnt(0), vtvec(), vmevec(), vvrvec()
// コンストラクタ
{
  vg = opt.g();
  vnc = vg * (vg - 1) / 2;
  vstativec.resize(vg + 1);  vstattvec.resize(vg);
  vsmat.resize(vnc + 1, vnc + 1);
  vtvec.resize(vnc + 1, 0.0);
  vmevec.resize(vnc + 1, 0.0);  vvrvec.resize(vnc + 1, 0.0);
}

void varclass::set_stat(const option&opt)
// 正規乱数を発生して基礎統計量を集計する
{
  for (int j = 0; j < vg + 1; ++j) vstativec[j].clear();
  for (int j = 0; j < vg; ++j)
  {
    int nj = opt.n(j);
    double mj = opt.m(j), sdj = opt.sd(j);
    for (int k = 0; k < nj; ++k) vstativec[j].sigma(randn(mj, sdj));
  }
  for (int j = 0; j < vg; ++j) vstativec[vg] += vstativec[j];
}

double varclass::cal_anova(double& vr) const // 分散分析を行う
{
  vr = 0.0;
  double sa = 0.0;
  for (int j = 0; j < vg; ++j)
  {
    int nj = vstativec[j].n();
    double mj = vstativec[j].mean();
    sa += nj * mj * mj;
  }
  double st = vstativec[vg].ss();
  int nt = vstativec[vg].n();
  double mt = vstativec[vg].mean();
  sa -= nt * mt * mt;
  double sr = st - sa;
  int dft = nt - 1, dfa = vg - 1, dfr = nt - vg;
  const double Seps = DBL_EPSILON * fmax(FDmin, st);
  if (sa < Seps || sr < Seps || dfr < 1) return -1.0; // 検定できず
  vr = sr / dfr;
  double f = (sa / dfa) / vr;
  return ftop(f, dfa, dfr);     // 有意確率を返す
}

bool varclass::cal_mcomp(bool& fsig, bool& bsig, bool& ssig, double vr, double alpha)
// 多重比較を行う
{
  int nt = vstativec[vg].n();
  int dfa = vg - 1, dfr = nt - vg;
  fsig = false;  bsig = false;  ssig = false;
  vtvec[0] = 1.0;
  int i = 1;
  for (int j = 0; j < vg - 1; ++j)
  {
    int nj = vstativec[j].n();
    double mj = vstativec[j].mean();
    for (int k = j + 1; (bsig == false || ssig == false) && k < vg; ++k)
    {
      int nk = vstativec[k].n();
      double mk = vstativec[k].mean();
      double dm = mj - mk;
      double f = dm * dm / (1.0 / (double)nj + 1.0 / (double)nk) / vr;
      vtvec[i] = sgn(dm) * sqrt(f);
      ++i;
      double pr = ftop(f, 1, dfr);
      if (j == 0 && k == 1)     // Fisher型は1度だけ検定
      {
        if (fsig == false && pr < alpha) fsig = true;
      }
      pr = 1.0 - pow(1.0 - pr, vnc); // Bonferroni型
      if (bsig == false && pr < alpha) bsig = true;
      f = dm * dm / dfa / (1.0 / (double)nj + 1.0 / (double)nk) / vr;
      pr = ftop(f, dfa, dfr);   // Scheffe型
      if (ssig == false && pr < alpha) ssig = true;
    }
  }
  return fsig || bsig || ssig;  // どれか1でもtrueならtrue
}

void varclass::set(const option& opt) // 変数をセットする
{
  srand(opt.seed());
  for (int i = 0; i < opt.r(); ++i)
  {
    set_stat(opt);
    for (int j = 0; j < vg; ++j) vstattvec[j].sigma(vstativec[j].mean());
    double vr = 0.0;
    double pr = cal_anova(vr);
    if (pr < 0.0) continue;     // 検定できず
    if (pr < opt.a()) ++vnasig;
    else if (opt.am()) continue; // ANOVAが有意の時だけ多重比較実施
    bool fsig = false, bsig = false, ssig = false;
    cal_mcomp(fsig, bsig, ssig, vr, opt.a());
    if (fsig) ++vnfsig;
    if (bsig) ++vnbsig;
    if (ssig) ++vnssig;
    for (int i = 0; i <= vnc; ++i)
    {
      double ti = vtvec[i];
      for (int j = 0; j <= vnc; ++j) vsmat.add(i, j, ti * vtvec[j]);
    }
  }
  vnt = (int)(vsmat(0, 0));
  for (int i = 0; i < vnc - 1; ++i)
  {
    for (int j = i + 1; j < vnc; ++j) vsmat.assign(j, i, vsmat(i, j));
  }
  vsmat.sweep(0);
  for (int i = 0; i <= vnc; ++i)
  {
    vmevec[i] = vsmat(0, i);
    vvrvec[i] = (vnt > 1) ? vsmat(i, i) / (double)(vnt - 1) : 0.0;
  }
}

double varclass::b1popu(size_t idx) const // 標本平均値の母歪度を返す
{
  if (idx >= vstattvec.size()) return 0.0;
  if (sgn(vstattvec[idx].m2()) <= 0) return 0.0;
  return vstattvec[idx].m3() / vstattvec[idx].m2() / sqrt(vstattvec[idx].m2());
}

double varclass::b2popu(size_t idx) const // 標本平均値の母尖度を返す
{
  if (idx >= vstattvec.size()) return 0.0;
  if (sgn(vstattvec[idx].m2()) <= 0) return 0.0;
  return vstattvec[idx].m4() / vstattvec[idx].m2() / vstattvec[idx].m2();
}

class powsimclass               // powsimメインクラス
{
private:
  void put_title(ostream& fout, const option& opt); // タイトルを出力する
  void set_sk(string& skew, string& kur, double b1, double b2) const;
  // 母歪度と母尖度を判定して文字列を返す
public:
  powsimclass()                 // コンストラクタ
  {}
  ~powsimclass()                // デストラクタ
  {}
  void put(option& opt);        // シミュレーション結果を出力する
};

void powsimclass::put_title(ostream& fout, const option& opt)
// タイトルを出力する
{
  puttitle(fout, Title);
  fout << "\n";
  fout << "○有意水準=" << opt.a() << " 群数=" << opt.g() << "  繰り返し数=" << opt.r() << "\n";
  if (opt.am()) fout << "  分散分析が有意の時だけ多重比較を実施する\n";
  else fout << "  分散分析と多重比較を独立に実施する\n";
}

void powsimclass::set_sk(string& skew, string& kur, double b1, double b2) const
// 母歪度と母尖度を判定して文字列を返す
{
  skew.erase();  kur.erase();
  if (b1 < -1.0) skew = "右傾斜";
  else if (fabs(b1) <= 1.0) skew = "ほぼ対称";
  else skew = "左傾斜";
  if (b2 < 0.0) kur = "鈍峰";
  else if (0.0 <= b2 && b2 <= 6.0) kur = "ほぼ正規";
  else kur = "鋭峰";
}

void powsimclass::put(option& opt) // シミュレーション結果を出力する
{
  if (opt.check() != 0) throw option_error("実行時オプションエラー！");
  varclass var(opt);            // 変数オブジェクト
  var.set(opt);                 // 変数をセットする
  put_title(cout, opt);
  cout << "\n";
  for (int j = 0; j < opt.g(); ++j)
  {
    cout << "・群" << j << ":" << "例数=" << opt.n(j) << " 平均値=" << opt.m(j) << " 標準偏差=" << opt.sd(j);
    if (opt.v())
    {
      cout << "  ↓標本平均の基礎統計量\n";
      cout << "    ";
      putmsd(cout, var.n(j), var.mean(j), var.vr(j));
      string skew, kur;
      set_sk(skew, kur, var.b1popu(j), var.b2popu(j));
      cout << "    歪度√β1=" << var.b1popu(j) << " " << skew << "  尖度β2=" << var.b2popu(j) << " " << kur;
    }
    cout << "\n";
  }
  if (opt.v())                  // t値の基礎統計量と相関係数出力
  {
    cout << "\n";
    putmsdvec(cout, var.nt(), var.mevec(), var.vrvec());
    putmat(cout, "・t値の相関行列", matr, var.nt(), var.smat());
  }
  double po = (double)var.nasig() / (double)(opt.r());
  cout << "\n・一元配置分散分析の検出力    =" << po << "(" << var.nasig() << "/" << opt.r() <<", β=" << 1.0 - po << ")\n";
  po = (double)var.nfsig() / (double)(opt.r());
  cout << "  Fisher型多重比較の検出力    =" << po << "(" << var.nfsig() << "/" << opt.r() <<", β=" << 1.0 - po << ")\n";
  po = (double)var.nbsig() / (double)(opt.r());
  cout << "  Bonferroni型多重比較の検出力=" << po << "(" << var.nbsig() << "/" << opt.r() <<", β=" << 1.0 - po << ")\n";
  po = (double)var.nssig() / (double)(opt.r());
  cout << "  Scheffe型多重比較の検出力   =" << po << "(" << var.nssig() << "/" << opt.r() <<", β=" << 1.0 - po << ")\n";
  cout << "\n" << endl;
}

int main(int argc, char* argv[]) // メインルーチン
{
  powsimclass powsim;           // メインクラス変数
  option opt(argv[0]);          // オプション
  try                           // 例外監視
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理モードフラッグ
    for (int i = 1; i < argc; ++i)
    {
      if (*(argv[i]) == CHRopt) opt.set(argv[i] + 1);
    }
    powsim.put(opt);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "内容不明なエラー発生！");
    return 255;
  }
  return 0;
}
